<?php
/**
 * Copyright © 2019 Magmodules.eu. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Magmodules\Sooqr\Model\Collection;

use Magento\Eav\Model\Config as EavConfig;
use Magento\Framework\App\ResourceConnection;
use Magento\Catalog\Model\ResourceModel\Product\Collection as ProductCollection;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory as ProductCollectionFactory;
use Magento\Catalog\Model\ResourceModel\Product\Attribute\CollectionFactory as ProductAttributeCollectionFactory;
use Magento\Catalog\Model\Indexer\Product\Flat\State as FlatState;
use Magento\Catalog\Model\Indexer\Product\Flat\StateFactory;
use Magento\CatalogInventory\Helper\Stock as StockHelper;
use Magmodules\Sooqr\Helper\Product as ProductHelper;
use Magmodules\Sooqr\Helper\General as GeneralHelper;
use Magento\Review\Model\ReviewFactory;
use Magento\Framework\Exception\LocalizedException;

/**
 * Class Products
 *
 * @package Magmodules\Sooqr\Model\Collection
 */
class Products
{

    /**
     * @var ProductCollectionFactory
     */
    private $productCollectionFactory;

    /**
     * @var ProductAttributeCollectionFactory
     */
    private $productAttributeCollectionFactory;

    /**
     * @var EavConfig
     */
    private $eavConfig;

    /**
     * @var StateFactory
     */
    private $productFlatState;

    /**
     * @var StockHelper
     */
    private $stockHelper;

    /**
     * @var GeneralHelper
     */
    private $generalHelper;

    /**
     * @var ProductHelper
     */
    private $productHelper;

    /**
     * @var ReviewFactory
     */
    private $reviewFactory;

    /**
     * @var ResourceConnection
     */
    private $resource;

    /**
     * Products constructor.
     *
     * @param ProductCollectionFactory          $productCollectionFactory
     * @param ProductAttributeCollectionFactory $productAttributeCollectionFactory
     * @param EavConfig                         $eavConfig
     * @param StockHelper                       $stockHelper
     * @param GeneralHelper                     $generalHelper
     * @param ProductHelper                     $productHelper
     * @param ReviewFactory                     $reviewFactory
     * @param StateFactory                      $productFlatState
     * @param ResourceConnection                $resource
     */
    public function __construct(
        ProductCollectionFactory $productCollectionFactory,
        ProductAttributeCollectionFactory $productAttributeCollectionFactory,
        EavConfig $eavConfig,
        StockHelper $stockHelper,
        GeneralHelper $generalHelper,
        ProductHelper $productHelper,
        ReviewFactory $reviewFactory,
        StateFactory $productFlatState,
        ResourceConnection $resource
    ) {
        $this->productCollectionFactory = $productCollectionFactory;
        $this->productAttributeCollectionFactory = $productAttributeCollectionFactory;
        $this->eavConfig = $eavConfig;
        $this->productFlatState = $productFlatState;
        $this->productHelper = $productHelper;
        $this->generalHelper = $generalHelper;
        $this->stockHelper = $stockHelper;
        $this->reviewFactory = $reviewFactory;
        $this->resource = $resource;
    }

    /**
     * @param $config
     * @param $page
     * @param $productIds
     *
     * @return ProductCollection
     * @throws LocalizedException
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     */
    public function getCollection($config, $page, $productIds)
    {

        $filters = $config['filters'];
        $attributes = $this->getAttributes($config['attributes']);
        $productFlatState = $this->getProductFlatState($config);

        /** @var ProductCollection $collection */
        $collection = $this->productCollectionFactory->create(['catalogProductFlatState' => $productFlatState]);
        $collection->addAttributeToSelect($attributes)
            ->addStoreFilter($config['store_id'])
            ->addUrlRewrite()
            ->setOrder('entity_id', 'ASC');

        if (!empty($filters['visibility'])) {
            $collection->addAttributeToFilter('visibility', ['in' => $filters['visibility']]);
        }

        if (($filters['limit'] > 0) && empty($productId)) {
            $collection->setPage($page, $filters['limit'])->getCurPage();
        }

        if (!empty($filters['type_id'])) {
            $collection->addAttributeToFilter('type_id', ['in' => $filters['type_id']]);
        }

        if (!empty($productIds)) {
            $collection->addAttributeToFilter('entity_id', ['in' => $productIds]);
        }

        if (!empty($filters['category_ids'])) {
            if (!empty($filters['category_type'])) {
                $collection->addCategoriesFilter([$filters['category_type'] => $filters['category_ids']]);
            }
        }

        $this->joinCatalogInventoryLeft($collection, $config);
        $this->addFilters($filters, $collection);
        $this->joinPriceIndexLeft($collection, $config['website_id']);
        $this->appendReviews($config, $collection);

        return $collection;
    }

    /**
     * @param $selectedAttrs
     *
     * @return array
     */
    private function getAttributes($selectedAttrs)
    {
        $attributes = $this->getProductAttributes();
        foreach ($selectedAttrs as $selectedAtt) {
            if (!empty($selectedAtt['source'])) {
                if (empty($selectedAtt['inventory'])) {
                    $attributes[] = $selectedAtt['source'];
                }
            }
            if (!empty($selectedAtt['multi']) && is_array($selectedAtt['multi'])) {
                foreach ($selectedAtt['multi'] as $attribute) {
                    $attributes[] = $attribute['source'];
                }
            }
            if (!empty($selectedAtt['main'])) {
                $attributes[] = $selectedAtt['main'];
            }
        }

        return array_unique($attributes);
    }

    /**
     * @return array
     */
    private function getProductAttributes()
    {
        return [
            'entity_id',
            'image',
            'price',
            'price_type',
            'special_price',
            'special_from_date',
            'special_to_date',
            'status',
            'tax_class_id',
            'weight',
            'product_has_weight',
            'quantity_and_stock_status'
        ];
    }

    /**
     * @param $config
     *
     * @return FlatState
     */
    private function getProductFlatState($config)
    {
        if ($config['flat']) {
            return $this->productFlatState->create(['isAvailable' => true]);
        }

        return $this->productFlatState->create(['isAvailable' => false]);
    }

    /**
     * @param ProductCollection $collection
     * @param array             $config
     */
    private function joinCatalogInventoryLeft($collection, $config)
    {
        $joinCond = join(
            ' AND ',
            ['cataloginventory.product_id = e.entity_id', 'cataloginventory.website_id = 0']
        );
        $tableName = ['cataloginventory' => $collection->getTable('cataloginventory_stock_item')];
        $collection->getSelect()->joinLeft(
            $tableName,
            $joinCond,
            array_combine($config['inventory']['attributes'], $config['inventory']['attributes'])
        );
    }

    /**
     * @param                                                         $filters
     * @param ProductCollection                                       $collection
     * @param string                                                  $type
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     *
     * @throws LocalizedException
     */
    public function addFilters($filters, $collection, $type = 'simple')
    {
        $cType = [
            'eq'   => '=',
            'neq'  => '!=',
            'gt'   => '>',
            'gteq' => '>=',
            'lt'   => '<',
            'lteg' => '<='
        ];

        $collection->addAttributeToFilter('status', 1);

        $attributeModel = $this->eavConfig->getAttribute('catalog_product', 'sooqr_exclude');
        if ($attributeModel->getId()) {
            $filters['advanced'][] = [
                'attribute'    => 'sooqr_exclude',
                'condition'    => 'neq',
                'value'        => 1,
                'product_type' => 'exclude'
            ];
        }

        foreach ($filters['advanced'] as $filter) {
            $attribute = $filter['attribute'];
            $condition = $filter['condition'];
            $value = $filter['value'];
            $productFilterType = $filter['product_type'];
            $filterExpr = [];

            if ($type == 'simple' && $productFilterType == 'parent') {
                continue;
            }
            if ($type == 'parent' && $productFilterType == 'simple') {
                continue;
            }

            $attributeModel = $this->eavConfig->getAttribute('catalog_product', $attribute);
            if (!$attributeModel->getAttributeCode()) {
                continue;
            }

            $frontendInput = $attributeModel->getFrontendInput();
            if (($frontendInput == 'select' || $frontendInput == 'multiselect') && $frontendInput != 'boolean') {
                $options = $attributeModel->getSource()->getAllOptions();
                if (strpos($value, ',') !== false) {
                    $values = [];
                    $value = explode(',', $value);
                    foreach ($value as $v) {
                        $valueId = array_search(trim($v), array_column($options, 'label'));
                        if ($valueId) {
                            $values[] = $options[$valueId]['value'];
                        }
                    }
                    $value = implode(',', $values);
                } else {
                    $valueId = array_search($value, array_column($options, 'label'));
                    if ($valueId) {
                        $value = $options[$valueId]['value'];
                    }
                }
            }

            if ($attribute == 'quantity_and_stock_status') {
                if ((isset($cType[$condition])) && is_numeric($value)) {
                    $collection->getSelect()->where(
                        'cataloginventory_stock_item.qty ' . $cType[$condition] . ' ' . $value
                    );
                }
                continue;
            }

            if ($attribute == 'min_sale_qty') {
                if ((isset($cType[$condition])) && is_numeric($value)) {
                    $collection->getSelect()->where(
                        'cataloginventory_stock_item.min_sale_qty ' . $cType[$condition] . ' ' . $value
                    );
                }
                continue;
            }

            switch ($condition) {
                case 'nin':
                    if (strpos($value, ',') !== false) {
                        $value = explode(',', $value);
                    }
                    $filterExpr[] = ['attribute' => $attribute, $condition => $value];
                    $filterExpr[] = ['attribute' => $attribute, 'null' => true];
                    break;
                case 'in':
                    if (strpos($value, ',') !== false) {
                        $value = explode(',', $value);
                    }
                    $filterExpr[] = ['attribute' => $attribute, $condition => $value];
                    break;
                case 'neq':
                    $filterExpr[] = ['attribute' => $attribute, $condition => $value];
                    $filterExpr[] = ['attribute' => $attribute, 'null' => true];
                    break;
                case 'empty':
                    $filterExpr[] = ['attribute' => $attribute, 'null' => true];
                    break;
                case 'not-empty':
                    $filterExpr[] = ['attribute' => $attribute, 'notnull' => true];
                    break;
                case 'gt':
                case 'gteq':
                case 'lt':
                case 'lteq':
                    if (is_numeric($value)) {
                        $filterExpr[] = ['attribute' => $attribute, $condition => $value];
                    }
                    break;
                default:
                    $filterExpr[] = ['attribute' => $attribute, $condition => $value];
                    break;
            }

            if (!empty($filterExpr)) {
                if ($productFilterType == 'parent') {
                    $filterExpr[] = ['attribute' => 'type_id', 'eq' => 'simple'];
                    /** @noinspection PhpParamsInspection */
                    $collection->addAttributeToFilter($filterExpr, '', 'left');
                } elseif ($productFilterType == 'simple') {
                    $filterExpr[] = ['attribute' => 'type_id', 'neq' => 'simple'];
                    /** @noinspection PhpParamsInspection */
                    $collection->addAttributeToFilter($filterExpr, '', 'left');
                } elseif ($productFilterType == 'exclude') {
                    /** @noinspection PhpParamsInspection */
                    $collection->addAttributeToFilter($filterExpr, '', 'left');
                } else {
                    /** @noinspection PhpParamsInspection */
                    $collection->addAttributeToFilter($filterExpr);
                }
            }
        }

        if (!empty($filters['stock'])) {
            $this->stockHelper->addInStockFilterToCollection($collection);
            $collection->setFlag('has_stock_status_filter', true);
        } else {
            $collection->setFlag('has_stock_status_filter', false);
        }
    }

    /**
     * @param ProductCollection $collection
     * @param                   $websiteId
     */
    public function joinPriceIndexLeft($collection, $websiteId)
    {
        $joinCond = join(
            ' AND ',
            [
                'price_index.entity_id = e.entity_id',
                'price_index.website_id = ' . $websiteId,
                'price_index.customer_group_id = 0'
            ]
        );
        $colls = ['final_price', 'min_price', 'max_price'];
        $tableName = ['price_index' => $collection->getTable('catalog_product_index_price')];
        $collection->getSelect()->joinLeft($tableName, $joinCond, $colls);
    }

    /**
     * @param array             $config
     * @param ProductCollection $collection
     */
    private function appendReviews($config, $collection)
    {
        if (!$config['reviews']) {
            return;
        }

        $this->reviewFactory->create()->appendSummary($collection);
    }

    /**
     * @param       $parentRelations
     * @param array $config
     *
     * @return ProductCollection
     * @throws LocalizedException
     */
    public function getParents($parentRelations, $config)
    {
        if (!empty($parentRelations)) {
            $filters = $config['filters'];
            $attributes = $this->getAttributes($config['attributes']);
            $productFlatState = $this->getProductFlatState($config);

            /** @var ProductCollection $collection */
            $collection = $this->productCollectionFactory->create(['catalogProductFlatState' => $productFlatState]);
            $collection->addAttributeToSelect($attributes)
                ->addAttributeToFilter('entity_id', ['in' => array_values($parentRelations)])
                ->addStoreFilter($config['store_id'])
                ->addUrlRewrite();

            if (!empty($filters['category_ids'])) {
                if (!empty($filters['category_type'])) {
                    $collection->addCategoriesFilter([$filters['category_type'] => $filters['category_ids']]);
                }
            }

            if (!empty($filters['visibility_parent'])) {
                $collection->addAttributeToFilter('visibility', ['in' => $filters['visibility_parent']]);
            }

            $this->joinCatalogInventoryLeft($collection, $config);
            $this->addFilters($filters, $collection, 'parent');
            $this->joinPriceIndexLeft($collection, $config['website_id']);

            return $collection->load();
        }
    }

    /**
     * Direct Database Query to get total records of collection with filters.
     *
     * @param ProductCollection $productCollection
     *
     * @return int
     */
    public function getCollectionCountWithFilters($productCollection)
    {
        $selectCountSql = $productCollection->getSelectCountSql();
        $connection = $this->resource->getConnection(ResourceConnection::DEFAULT_CONNECTION);
        return $connection->fetchOne($selectCountSql);
    }
}
